<?php

namespace PAR\Service;

use DASPRiD\EnumTest\NullValueTest;
use PAR\Factory\BusinessPartnerFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use PAR\Service\FileServerService;

class BusinessPartnerService {

    protected $connection;
    protected $businessPartnerFactory;
    protected $environment;
    protected $newCode;
    protected $FileServerService;
    protected $typeOfPartner = array(
        "CPF" => "F",
        "CNPJ" => "J",
        "LIVRE" => "I",
        "CODIGO" => "O"
    );

    public function __construct(BusinessPartnerFactory $businessPartnerFactory, Environment $environment, NewCode $newCode, EntityManager $entityManager, FileServerService $FileServerService) {
        $this->businessPartnerFactory = $businessPartnerFactory;
        $this->connection             = $entityManager->getConnection();
        $this->environment            = $environment;
        $this->newCode                = $newCode;
        $this->FileServerService      = $FileServerService;
    }


    public function getBusinessPartner() {
        return $this->businessPartnerFactory->getBusinessPartner();
    }

    public function getOperator() {
        return $this->businessPartnerFactory->getOperator();
    }

    public function getPartnerClassification() {
        return $this->businessPartnerFactory->getPartnerClassification();
    }

    public function getTypeOfPartner() {
        return $this->businessPartnerFactory->getTypeOfPartner();
    }

    public function getMarketSegment() {
        return $this->businessPartnerFactory->getMarketSegment();
    }

    public function getBranchOfActivity() {
        return $this->businessPartnerFactory->getBranchOfActivity();
    }

     public function getTypeOfActivity() {
        return $this->businessPartnerFactory->getTypeOfActivity();
    }

    public function getSpecificBusinessPartner($parametros) {
        return $this->businessPartnerFactory->getSpecificBusinessPartner($parametros);
    }

    public function insertNewBusinessPartner($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $NRPARCNEGOCIO = $this->newCode->getCodeByTableName("PARCNEGOCIO", $NRORG, 1, 10, true);

            $dataParam = array(
                "NRPARCNEGOCIO"      => $NRPARCNEGOCIO,
                "NRORG"              => $NRORG,
                "CDSEGMER"           => $parametros['CDSEGMER'],
                "NMPRINCIPALPARC"    => $parametros['NMPRINCIPALPARC'],
                "NMSECUNDARIPARC"    => $parametros['NMSECUNDARIPARC'],
                "DTNASCIFUNDPARC"    => ($parametros['DTNASCIFUNDPARC'] == '') ? null : $parametros['DTNASCIFUNDPARC'],
                "NRINSCRICAOPARC"    => $parametros['NRINSCRICAOPARC'],
                "NRORGINCLUSAO"      => $NRORG,
                "CDOPERINCLUSAO"     => $parametros['CDOPERADOR'],
                "CDTIPOPARCPRINCIPAL"=> $parametros['CDTIPOPARCPRINCIPAL'],
                "CDTIPOINSCRICAO"    => $parametros['CDTIPOINSCRICAO'],
                "IDPESSOAFISICA"     => $parametros['IDPESSOAFISICA'],
                "IDINSTITUICAO"      => $parametros['IDINSTITUICAO'],
                "CDATIVCLI"          => $parametros['CDATIVCLI'],
                "NRSEQTPATIVCLI"     => $parametros['NRSEQTPATIVCLI'],
                "CDOPERRESPONSAVEL"  => $parametros['CDOPERRESPONSAVEL'],
                "NRTPCLASSIFICAPARC" => $parametros['NRTPCLASSIFICAPARC'],
                "CDINSCESTA"         => $parametros['CDINSCESTA'],
                "CDINSCMUNI"         => $parametros['CDINSCMUNI'],
            );

            $this->businessPartnerFactory->insertNewBusinessPartner($dataParam);

            $NRENDERECOPARC = $this->newCode->getCodeByTableName("ENDERECOPARC", $NRORG, 1, 10, true);

            $dataAddress = array(
                "NRENDERECOPARC"   => $NRENDERECOPARC,
                "NRPARCNEGOCIO"    => $NRPARCNEGOCIO,
                "CDTIPOENDERECO"   => $parametros['CDTIPOENDERECO'],
                "NRORG"            => $NRORG,
                "NRORGINCLUSAO"    => $NRORG,
                "CDOPERINCLUSAO"   => $parametros['CDOPERADOR'],
                "CDPAIS"           => $parametros['CDPAIS'],
                "SGESTADO"         => $parametros['SGESTADO'],
                "CDMUNICIPIO"      => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"         => $parametros['CDBAIRRO'],
                "NMBAIRROENDERECO" => $parametros['NMBAIRROENDERECO'],
                "DSCOMPLEENDERECO" => $parametros['DSCOMPLEENDERECO'],
                "NRCEPENDERECO"    => $parametros['NRCEPENDERECO'],
                "DSENDERECO"       => $parametros['DSENDERECO'],
                "NRIMOVELENDERECO" => $parametros['NRIMOVELENDERECO'],
                "DSREFERENCIAENDE" => $parametros['DSREFERENCIAENDE']
            );

            $this->businessPartnerFactory->insertNewAddress($dataAddress);

            foreach($parametros['CDTIPOPARCEIRO'] as $typePartner) {

                $this->insertPARCERIA($parametros, $NRPARCNEGOCIO, $NRORG, $typePartner);

                switch ($typePartner) {
                    case 'CLIENTE':
                        $this->insertCLIENTE($parametros, $NRPARCNEGOCIO, $NRORG);
                        break;

                    case 'FORNECEDOR':
                        $this->insertFORNECEDOR($parametros, $NRPARCNEGOCIO, $NRORG);
                        break;

                    case 'VENDEDOR':
                        $this->insertVENDEDOR($parametros, $NRPARCNEGOCIO, $NRORG);
                        break;

                    case 'PROSPECT':
                        $this->insertPROSPECT($parametros, $NRPARCNEGOCIO, $NRORG);
                        break;
                }
            }

            $this->connection->commit();
            $response = 'OK';

        }
        catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
            //$response = $e->getMessage();
        }

        return $response;
    }

    public function insertPARCERIA($parametros, $NRPARCNEGOCIO, $NRORG, $typePartner) {
        $NRPARCERIA = $this->newCode->getCodeByTableName("PARCERIA", $NRORG, 1, 10, true);

        $dataTypes = array (
            "NRPARCERIA"     => $NRPARCERIA,
            "NRPARCNEGOCIO"  => $NRPARCNEGOCIO,
            "CDTIPOPARCEIRO" => $typePartner,
            "NRORG"          => $NRORG,
            "NRORGINCLUSAO"  => $NRORG,
            "IDATIVO"        => 'S',
            "CDOPERINCLUSAO" => $parametros['CDOPERADOR'],
            "NRPARCORIGEM"   => $NRPARCNEGOCIO
        );
        $this->businessPartnerFactory->insertPartnership($dataTypes);
    }

    public function insertCLIENTE($parametros, $NRPARCNEGOCIO, $NRORG) {
        $arrayCLIENTE = array(
            "CDCLIENTE"     => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']].$parametros['NRINSCRICAOPARC'],
            "IDTPIJURCLIE"  => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURCLIE"  => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCCLIE"  => $parametros['NMPRINCIPALPARC'],
            "NMFANTCLIE"    => $parametros['NMSECUNDARIPARC'],
            "NRORG"         => $NRORG,
            "NRORGINCLUSAO" => $NRORG,
            "CDOPERINCLUSAO"=> $parametros['CDOPERADOR'],
            "NRPARCNEGOCIO" => $NRPARCNEGOCIO
        );
        $this->businessPartnerFactory->insertCLIENTE($arrayCLIENTE);
    }

    public function insertFORNECEDOR($parametros, $NRPARCNEGOCIO, $NRORG) {
        $arrayFORNECEDOR = array(
            "CDFORNECED"    => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']].$parametros['NRINSCRICAOPARC'],
            "IDTPIJURFORN"  => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURFORN"  => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCFORN"  => $parametros['NMPRINCIPALPARC'],
            "NMFANTFORN"    => $parametros['NMSECUNDARIPARC'],
            "NRORG"         => $NRORG,
            "NRORGINCLUSAO" => $NRORG,
            "CDOPERINCLUSAO"=> $parametros['CDOPERADOR'],
            "NRPARCNEGOCIO" => $NRPARCNEGOCIO
        );
        $this->businessPartnerFactory->insertFORNECEDOR($arrayFORNECEDOR);
    }

    public function insertVENDEDOR($parametros, $NRPARCNEGOCIO, $NRORG) {
        $CDVENDEDOR = $this->newCode->getCodeByTableName("VENDEDOR", $NRORG, 1, 10, true);

        $arrayVENDEDOR = array(
            "CDVENDEDOR"    => $CDVENDEDOR,
            "IDTPIJURVEN"   => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURVEN"   => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCVEN"   => $parametros['NMPRINCIPALPARC'],
            "NMFANVEN"      => $parametros['NMSECUNDARIPARC'],
            "NRORG"         => $NRORG,
            "NRORGINCLUSAO" => $NRORG,
            "CDOPERINCLUSAO"=> $parametros['CDOPERADOR'],
            "NRPARCNEGOCIO" => $NRPARCNEGOCIO
        );

        $this->businessPartnerFactory->insertVENDEDOR($arrayVENDEDOR);
    }

    public function insertPROSPECT($parametros, $NRPARCNEGOCIO, $NRORG) {
        $CDPROSPECT = $this->newCode->getCodeByTableName("PROSPECT", $NRORG, 1, 10, true);

        $arrayPROSPECT = array(
            "CDPROSPECT"    => $CDPROSPECT,
            "IDTPIJURPROS"  => $this->typeOfPartner[$parametros['CDTIPOINSCRICAO']],
            "NRINSJURPROS"  => $parametros['NRINSCRICAOPARC'],
            "NMRAZSOCPROS"  => $parametros['NMPRINCIPALPARC'],
            "NMFANTPROS"    => $parametros['NMSECUNDARIPARC'],
            "NRORG"         => $NRORG,
            "NRORGINCLUSAO" => $NRORG,
            "CDOPERINCLUSAO"=> $parametros['CDOPERADOR'],
            "NRPARCORIGEM"  => $NRPARCNEGOCIO,
            "NRPARCNEGOCIO" => $NRPARCNEGOCIO,
            "IDATIVO"       => $parametros['IDATIVO']
        );

        $this->businessPartnerFactory->insertPROSPECT($arrayPROSPECT);
    }

    public function updateSpecificBusinessPartner($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $dataParam = array(
                "NRPARCNEGOCIO"       => $parametros['NRPARCNEGOCIO'],
                "CDSEGMER"            => $parametros['CDSEGMER'],
                "NRORGULTATU"         => $NRORG,
                "CDOPERULTATU"        => $parametros['CDOPERADOR'],
                "NMPRINCIPALPARC"     => $parametros['NMPRINCIPALPARC'],
                "NMSECUNDARIPARC"     => $parametros['NMSECUNDARIPARC'],
                "IDATIVO"             => $parametros['IDATIVO'],
                "DTNASCIFUNDPARC"     => ($parametros['DTNASCIFUNDPARC'] == '') ? null : $parametros['DTNASCIFUNDPARC'],
                "NRINSCRICAOPARC"     => $parametros['NRINSCRICAOPARC'],
                "CDTIPOPARCPRINCIPAL" => $parametros['CDTIPOPARCPRINCIPAL'],
                "CDTIPOINSCRICAO"     => $parametros['CDTIPOINSCRICAO'],
                "CDATIVCLI"           => $parametros['CDATIVCLI'],
                "IDPESSOAFISICA"      => $parametros['IDPESSOAFISICA'],
                "IDINSTITUICAO"       => $parametros['IDINSTITUICAO'],
                "NRSEQTPATIVCLI"      => $parametros['NRSEQTPATIVCLI'],
                "CDOPERRESPONSAVEL"   => $parametros['CDOPERRESPONSAVEL'],
                "NRTPCLASSIFICAPARC"  => $parametros['NRTPCLASSIFICAPARC'],
                "CDINSCESTA"          => $parametros['CDINSCESTA'],
                "CDINSCMUNI"          => $parametros['CDINSCMUNI']
            );

            $this->businessPartnerFactory->updateSpecificBusinessPartner($dataParam);
            $types = null;
            $IDATIVO = null;

            foreach($parametros['CDTIPOPARCEIRO'] as $type) {
                $types = $types."'".$type."'".', ';
            }

            $data = array(
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                "CDTIPOPARCEIRO" => substr($types, 0, -2)
            );

            $result = $this->businessPartnerFactory->searchForExistingPartnerTypes($data);

            // retorna tipos de parceiros de negócios que estão com IDATIVO = 'N', para que seja ativado novamente
            $result2 = $this->businessPartnerFactory->searchForInactivatePartnerTypes($parametros['NRPARCNEGOCIO']);

            if (count($result) > 0) {
                $IDATIVO = 'N';
                $this->activateOrInactivate($result, $parametros, $NRORG, $IDATIVO);
            }
            if (count($result2) > 0) {
                $IDATIVO = 'S';
                $this->activateOrInactivate($result2, $parametros, $NRORG, $IDATIVO);
            }

            foreach ($parametros['CDTIPOPARCEIRO'] as $type) {

                $data = array(
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "CDTIPOPARCEIRO" => $type
                );

                $result3 = $this->businessPartnerFactory->searchPartnesTypes($data);

                if (count($result3) == 0) {
                    $this->insertPARCERIA($parametros, $parametros['NRPARCNEGOCIO'], $NRORG, $type);

                    switch ($type) {
                        case 'CLIENTE':
                            $this->insertCLIENTE($parametros, $parametros['NRPARCNEGOCIO'], $NRORG);
                            break;

                        case 'FORNECEDOR':
                            $this->insertFORNECEDOR($parametros, $parametros['NRPARCNEGOCIO'], $NRORG);
                            break;

                        case 'VENDEDOR':
                            $this->insertVENDEDOR($parametros, $parametros['NRPARCNEGOCIO'], $NRORG);
                            break;

                        case 'PROSPECT':
                            $this->insertPROSPECT($parametros, $parametros['NRPARCNEGOCIO'], $NRORG);
                            break;
                    }
                }
            }

            $this->connection->commit();
        }
        catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function activateOrInactivate($result, $parametros, $NRORG, $IDATIVO) {
        foreach($result as $type) {

            $this->inactivateOrActivatePartnerType($parametros, $NRORG, $type['CDTIPOPARCEIRO'], $IDATIVO);

            switch ($type['CDTIPOPARCEIRO']) {
                case 'CLIENTE':
                    $this->updateCLIENTE($parametros, $NRORG, $IDATIVO);
                    break;

                case 'FORNECEDOR':
                    $this->updateFORNECEDOR($parametros, $NRORG, $IDATIVO);
                    break;

                case 'VENDEDOR':
                    $this->updateVENDEDOR($parametros, $NRORG, $IDATIVO);
                    break;

                case 'PROSPECT':
                    $this->updatePROSPECT($parametros, $NRORG, $IDATIVO);
                    break;
            }
        }
    }

    public function inactivateOrActivatePartnerType($parametros, $NRORG, $type, $IDATIVO) {
        $data = array(
            "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
            "NRORGULTATU"    => $NRORG,
            "CDOPERULTATU"   => $parametros['CDOPERADOR'],
            "CDTIPOPARCEIRO" => $type,
            "NRORG"          => $NRORG,
            "IDATIVO"        => $IDATIVO
        );

        $this->businessPartnerFactory->inactivateOrActivatePartnerType($data);
    }

    public function updateCLIENTE($parametros, $NRORG, $IDATIVO) {
        $data = array(
            "NRPARCNEGOCIO" => $parametros['NRPARCNEGOCIO'],
            "NRORGULTATU"   => $NRORG,
            "CDOPERULTATU"  => $parametros['CDOPERADOR'],
            "IDCLIEATIV"    => $IDATIVO,
            "IDATIVO"      => $IDATIVO
        );

        $this->businessPartnerFactory->updateCLIENTE($data);
    }

    public function updateFORNECEDOR($parametros, $NRORG, $IDATIVO) {
        $data = array(
            "NRPARCNEGOCIO" => $parametros['NRPARCNEGOCIO'],
            "NRORGULTATU"   => $NRORG,
            "CDOPERULTATU"  => $parametros['CDOPERADOR'],
            "IDATIVO"      => $IDATIVO
        );

        $this->businessPartnerFactory->updateFORNECEDOR($data);
    }

    public function updateVENDEDOR($parametros, $NRORG, $IDATIVO) {
        $data = array(
            "NRPARCNEGOCIO" => $parametros['NRPARCNEGOCIO'],
            "NRORGULTATU"   => $NRORG,
            "CDOPERULTATU"  => $parametros['CDOPERADOR'],
            "IDATIVO"      => $IDATIVO
        );

        $this->businessPartnerFactory->updateVENDEDOR($data);
    }

    public function updatePROSPECT($parametros, $NRORG, $IDATIVO) {
        $data = array(
            "NRPARCNEGOCIO" => $parametros['NRPARCNEGOCIO'],
            "NRORGULTATU"   => $NRORG,
            "CDOPERULTATU"  => $parametros['CDOPERADOR'],
            "IDATIVO"      => $IDATIVO
        );

        $this->businessPartnerFactory->updatePROSPECT($data);
    }

    //  ------------- FUNÇÕES  DE ENDEREÇO ------------------------
    public function getCountry() {
        return $this->businessPartnerFactory->getCountry();
    }

    public function getState($parametro) {
        return $this->businessPartnerFactory->getState($parametro);
    }

    public function getCity($parametro) {
        return $this->businessPartnerFactory->getCity($parametro);
    }

    public function getDistrict($parametro) {
        return $this->businessPartnerFactory->getDistrict($parametro);
    }

    public function findCodeDistrict($parametros) {
        return $this->businessPartnerFactory->findCodeDistrict($parametros);
    }

    public function getBusinessPartnerAddress($parametro) {
        return $this->businessPartnerFactory->getBusinessPartnerAddress($parametro);
    }

    public function getSpecificAddress($parametro) {
        return $this->businessPartnerFactory->getSpecificAddress($parametro);
    }

    public function insertNewAddress($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $NRENDERECOPARC = $this->newCode->getCodeByTableName("ENDERECOPARC", $NRORG, 1, 10, true);

            $dataParam = array(
                "NRENDERECOPARC"   => $NRENDERECOPARC,
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDTIPOENDERECO"   => $parametros['CDTIPOENDERECO'],
                "NRORG"            => $NRORG,
                "NRORGINCLUSAO"    => $NRORG,
                "CDOPERINCLUSAO"   => $parametros['CDOPERINCLUSAO'],
                "CDPAIS"           => $parametros['CDPAIS'],
                "SGESTADO"         => $parametros['SGESTADO'],
                "CDMUNICIPIO"      => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"         => $parametros['CDBAIRRO'],
                "NMBAIRROENDERECO" => $parametros['NMBAIRROENDERECO'],
                "DSCOMPLEENDERECO" => $parametros['DSCOMPLEENDERECO'],
                "NRCEPENDERECO"    => $parametros['NRCEPENDERECO'],
                "DSENDERECO"       => $parametros['DSENDERECO'],
                "NRIMOVELENDERECO" => $parametros['NRIMOVELENDERECO'],
                "DSREFERENCIAENDE" => $parametros['DSREFERENCIAENDE']
            );
            $this->businessPartnerFactory->insertNewAddress($dataParam);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteAddress($parametros) {
        $this->connection->beginTransaction();
        try{
            $this->businessPartnerFactory->deleteAddress($parametros);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateAddress($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $dataParam = array(
                "NRENDERECOPARC"   => $parametros['NRENDERECOPARC'],
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDTIPOENDERECO"   => $parametros['CDTIPOENDERECO'],
                "NRORGULTATU"      => $NRORG,
                "CDOPERULTATU"     => $parametros['CDOPERULTATU'],
                "CDPAIS"           => $parametros['CDPAIS'],
                "SGESTADO"         => $parametros['SGESTADO'],
                "CDMUNICIPIO"      => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"         => $parametros['CDBAIRRO'],
                "NMBAIRROENDERECO" => $parametros['NMBAIRROENDERECO'],
                "DSCOMPLEENDERECO" => $parametros['DSCOMPLEENDERECO'],
                "NRCEPENDERECO"    => $parametros['NRCEPENDERECO'],
                "DSENDERECO"       => $parametros['DSENDERECO'],
                "NRIMOVELENDERECO" => $parametros['NRIMOVELENDERECO'],
                "DSREFERENCIAENDE" => $parametros['DSREFERENCIAENDE']
            );
            $this->businessPartnerFactory->updateAddress($dataParam);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE comunicação ------------------------
    public function getWayOfCommunication($parametro) {
        return $this->businessPartnerFactory->getWayOfCommunication($parametro);
    }

    public function getListWaysOfCommunication() {
        return $this->businessPartnerFactory->getListWaysOfCommunication();
    }

    public function getSpecificWayOfCommunication($parametro) {
        return $this->businessPartnerFactory->getSpecificWayOfCommunication($parametro);
    }

    public function insertNewCommunicationData($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $NRCOMUNICAPARC = $this->newCode->getCodeByTableName("COMUNICAPARC", $NRORG, 1, 10, true);

            $dataParam = array(
                "NRCOMUNICAPARC"   => $NRCOMUNICAPARC,
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDFORMACOMU"      => $parametros['CDFORMACOMU'],
                "NRENDERECOPARC"   => null,
                "CDPREFIXCOMUPARC" => $parametros['CDPREFIXCOMUPARC'],
                "CDCOMUNICAPARC"   => $parametros['CDCOMUNICAPARC'],
                "NRORG"            => $NRORG,
                "DTULTATU"         => null,
                "NRORGINCLUSAO"    => $NRORG,
                "CDOPERINCLUSAO"   => $parametros['CDOPERINCLUSAO'],
                "NRORGULTATU"      => null,
                "CDOPERULTATU"     => null,
                "CDCOMPLECOMUPARC" => null,
                "DSCONTATO"        => null
            );
            $this->businessPartnerFactory->insertNewCommunicationData($dataParam);
            $this->connection->commit();
        }
        catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteWayOfCommunication($parametro) {
        $this->connection->beginTransaction();
        try {
            $this->businessPartnerFactory->deleteWayOfCommunication($parametro);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateWayOfCommunication($parametros) {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->environment->getNrOrg();
            $dataParam = array(
                "NRCOMUNICAPARC"   => $parametros['NRCOMUNICAPARC'],
                "NRPARCNEGOCIO"    => $parametros['NRPARCNEGOCIO'],
                "CDFORMACOMU"      => $parametros['CDFORMACOMU'],
                "NRENDERECOPARC"   => null,
                "CDPREFIXCOMUPARC" => $parametros['CDPREFIXCOMUPARC'],
                "CDCOMUNICAPARC"   => $parametros['CDCOMUNICAPARC'],
                "NRORGULTATU"      => $NRORG,
                "CDOPERULTATU"     => $parametros['CDOPERULTATU'],
                "CDCOMPLECOMUPARC" => null,
                "DSCONTATO"        => null
            );
            $this->businessPartnerFactory->updateWayOfCommunication($dataParam);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    //  ------------- FUNÇÕES  DE INTERLOCUTOR ------------------------

    public function getSectors() {
        return $this->businessPartnerFactory->getSectors();
    }

    public function getPositionsOfInterlocutors() {
        return $this->businessPartnerFactory->getPositionsOfInterlocutors();
    }

    public function getInterlocutors($parametro) {
        return $this->businessPartnerFactory->getInterlocutors($parametro);
    }

    public function getSpecificInterlocutor($parametro) {
        return $this->businessPartnerFactory->getSpecificInterlocutor($parametro);
    }

    public function insertNewInterlocutor($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $NRSEQINTERLOCUTOR = $this->newCode->getCodeByTableName("CRM_INTERLOCUTOR", $NRORG, 1, 10, true);

            $dataParam = array(
                "NRSEQINTERLOCUTOR" => $NRSEQINTERLOCUTOR,
                "NMINTERLOCUTOR"    => $parametros['NMINTERLOCUTOR'],
                "NRPARCNEGOCIO"     => $parametros['NRPARCNEGOCIO'],
                "EMAILINTERLOC"     => $parametros['EMAILINTERLOC'],
                "NRTELEFONE1"       => $parametros['NRTELEFONE1'],
                "NRTELEFONE2"       => $parametros['NRTELEFONE2'],
                "NRTELEFONE3"       => $parametros['NRTELEFONE3'],
                "NRTELEFONE4"       => $parametros['NRTELEFONE4'],
                "CDCARGOINTERLOC"   => $parametros['CDCARGOINTERLOC'],
                "DTNASCIMENTO"      => $parametros['DTNASCIMENTO'],
                "IDRECEBEPRESENTE"  => $parametros['IDRECEBEPRESENTE'],
                "CDPAIS"            => $parametros['CDPAIS'],
                "SGESTADO"          => $parametros['SGESTADO'],
                "CDMUNICIPIO"       => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"          => $parametros['CDBAIRRO'],
                "NRCEPINTERLOCUTOR" => $parametros['NRCEPINTERLOCUTOR'],
                "DSENDEINTERLOCUTOR"=> $parametros['DSENDEINTERLOCUTOR'],
                "DSCOMPINTERLOCUTOR"=> $parametros['DSCOMPINTERLOCUTOR'],
                "NRENDEINTERLOCUTOR"=> $parametros['NRENDEINTERLOCUTOR'],
                "NRORG"             => $NRORG,
                "NRORGINCLUSAO"     => $NRORG,
                "CDOPERINCLUSAO"    => $parametros['CDOPERINCLUSAO'],
                "CDSETOR"           => $parametros['CDSETOR'],
                "NRCPFINTERLOC"     => $parametros['NRCPFINTERLOC'],
                "IDTPSIGNATARIO"    => $parametros['IDTPSIGNATARIO'],
                "IDNOMEVINDODAAPI"  => "N"
            );
            $this->businessPartnerFactory->insertNewInterlocutor($dataParam);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateInterlocutor($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        // debugApi([$parametros]);

        try {
            $dataParam = array(
                "NRSEQINTERLOCUTOR" => $parametros['NRSEQINTERLOCUTOR'],
                "NMINTERLOCUTOR"    => $parametros['NMINTERLOCUTOR'],
                "NRPARCNEGOCIO"     => $parametros['NRPARCNEGOCIO'],
                "EMAILINTERLOC"     => $parametros['EMAILINTERLOC'],
                "NRTELEFONE1"       => $parametros['NRTELEFONE1'],
                "NRTELEFONE2"       => $parametros['NRTELEFONE2'],
                "NRTELEFONE3"       => $parametros['NRTELEFONE3'],
                "NRTELEFONE4"       => $parametros['NRTELEFONE4'],
                "CDCARGOINTERLOC"   => $parametros['CDCARGOINTERLOC'],
                "DTNASCIMENTO"      => $parametros['DTNASCIMENTO'],
                "IDRECEBEPRESENTE"  => $parametros['IDRECEBEPRESENTE'],
                "CDPAIS"            => $parametros['CDPAIS'],
                "SGESTADO"          => $parametros['SGESTADO'],
                "CDMUNICIPIO"       => $parametros['CDMUNICIPIO'],
                "CDBAIRRO"          => $parametros['CDBAIRRO'],
                "NRCEPINTERLOCUTOR" => $parametros['NRCEPINTERLOCUTOR'],
                "DSENDEINTERLOCUTOR"=> $parametros['DSENDEINTERLOCUTOR'],
                "DSCOMPINTERLOCUTOR"=> $parametros['DSCOMPINTERLOCUTOR'],
                "NRENDEINTERLOCUTOR"=> $parametros['NRENDEINTERLOCUTOR'],
                "NRORGULTATU"       => $NRORG,
                "CDOPERULTATU"      => $parametros['CDOPERULTATU'],
                "CDSETOR"           => $parametros['CDSETOR'],
                "NRCPFINTERLOC"     => $parametros['NRCPFINTERLOC'],
                "IDTPSIGNATARIO"    => $parametros['IDTPSIGNATARIO'],
                "IDNOMEVINDODAAPI"  => "N"
            );
            $this->businessPartnerFactory->updateInterlocutor($dataParam);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteInterlocutor($parametro) {
        $this->connection->beginTransaction();
        try {
            $this->businessPartnerFactory->deleteInterlocutor($parametro);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    //  ------------- FUNÇÕES  DE DADOS BANCÁRIOS  ------------------------
    public function getBankDetails($parametro) {
        return $this->businessPartnerFactory->getBankDetails($parametro);
    }

    public function getBanks() {
        return $this->businessPartnerFactory->getBanks();
    }

    public function getAgencies($parametro) {
        return $this->businessPartnerFactory->getAgencies($parametro);
    }

    public function getCheckingAccount($parametro) {
        return $this->businessPartnerFactory->getCheckingAccount($parametro);
    }

    public function getTypeOfCheckingAccount() {
        return $this->businessPartnerFactory->getTypeOfCheckingAccount();
    }

    public function getSpecificBankDetails($parametro) {
        return $this->businessPartnerFactory->getSpecificBankDetails($parametro);
    }

    public function getBranches() {
        return $this->businessPartnerFactory->getBranches();
    }

    public function getInvoiceSeries() {
        return $this->businessPartnerFactory->getInvoiceSeries();
    }

    public function insertNewBankDetails($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        try {
            $NRCONTCORRPARC = $this->newCode->getCodeByTableName("CONTCORRPARC", $NRORG, 1, 10, true);
            $dataParam = array(
                "NRCONTCORRPARC" => $NRCONTCORRPARC,
                "NRORG"          => $NRORG,
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                "CDBANCO"        => $parametros['CDBANCO'],
                "CDAGENCIA"      => $parametros['CDAGENCIA'],
                "CDCONTCORR"     => $parametros['CDCONTCORR'],
                "NRTPCONTCORR"   => $parametros['NRTPCONTCORR'],
                "NRORGINCLUSAO"  => $NRORG,
                "CDOPERINCLUSAO" => $parametros['CDOPERINCLUSAO']
            );
            $this->businessPartnerFactory->insertNewBankDetails($dataParam);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteBankDetails($parametro) {
        $this->connection->beginTransaction();
        try {
            $this->businessPartnerFactory->deleteBankDetails($parametro);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateBankDetails($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();

        try {
            $NRCONTCORRPARC = $this->newCode->getCodeByTableName("CONTCORRPARC", $NRORG, 1, 10, true);
            $dataParam = array(
                "NRCONTCORRPARC" => $parametros['NRCONTCORRPARC'],
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                "CDBANCO"        => $parametros['CDBANCO'],
                "CDAGENCIA"      => $parametros['CDAGENCIA'],
                "CDCONTCORR"     => $parametros['CDCONTCORR'],
                "NRTPCONTCORR"   => $parametros['NRTPCONTCORR'],
                "NRORGULTATU"  => $NRORG,
                "CDOPERULTATU" => $parametros['CDOPERULTATU']
            );
            $this->businessPartnerFactory->updateBankDetails($dataParam);
            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

     //  ------------- FUNÇÕES  DE PARÂMETROS DE FATURAMENTO  ------------------------

    public function getBillingParameter($parametro) {
        return $this->businessPartnerFactory->getBillingParameter($parametro);
    }

    public function billingParameter($parametros) {
        $NRORG = $this->environment->getNrOrg();
        $this->connection->beginTransaction();
        // debugApi([$parametros]);
        try {
            if ($parametros['NRPARAMFATURA'] == null) {
                $NRPARAMFATURA = $this->newCode->getCodeByTableName("PAR_PARAMFATURA", $NRORG, 1, 10, true);
                $dataParam = array(
                    "NRPARAMFATURA"  => $NRPARAMFATURA,
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "CDFILIALFATURA" => $parametros['CDFILIALFATURA'],
                    "CDSERIENF"      => $parametros['CDSERIENF'],
                    "CDEMPRESA"      => $parametros['CDEMPRESA'],
                    "CDINSCESTA"     => $parametros['CDINSCESTA'],
                    "CDINSCMUNI"     => $parametros['CDINSCMUNI'],
                    "NRORG"          => $NRORG,
                    "NRORGINCLUSAO"  => $NRORG,
                    "CDOPERINCLUSAO" => $parametros['CDOPERADOR']
                );
                $this->businessPartnerFactory->insertBillingParameter($dataParam);
            } else {
                $NRPARAMFATURA = $parametros['NRPARAMFATURA'];
                $dataParamUpdate = array(
                    "NRPARAMFATURA"  => $parametros['NRPARAMFATURA'],
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "CDFILIALFATURA" => $parametros['CDFILIALFATURA'],
                    "CDSERIENF"      => $parametros['CDSERIENF'],
                    "CDEMPRESA"      => $parametros['CDEMPRESA'],
                    "CDINSCESTA"     => $parametros['CDINSCESTA'],
                    "CDINSCMUNI"     => $parametros['CDINSCMUNI'],
                    "NRORGULTATU"    => $NRORG,
                    "CDOPERULTATU"   => $parametros['CDOPERADOR']
                );
                $this->businessPartnerFactory->updateBillingParameter($dataParamUpdate);
            }
            $this->connection->commit();
            return $NRPARAMFATURA;
        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    //  ------------- FUNÇÕES  DE logotipo   ------------------------
    public function sendImage($parametros) {
        $this->connection->beginTransaction();
        try {
            //Remove a imagem antiga
            $dataParam = array(
                "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO']
            );
            $caminhoAntigo = $this->getImagesLogotype($dataParam);
            if (!empty($caminhoAntigo[0]["DSDIRLOGOMARCA"])) {
                $this->FileServerService->deleteFile($caminhoAntigo[0]["DSDIRLOGOMARCA"]);
            }
            //Insere a imagem Nova
            $link = '';
            $row = $this->FileServerService->uploadFiles($parametros['DSDIRLOGOMARCA'], 'anexo_parceiro');
            foreach($row as $file) {
                $dataParam = array(
                    "NRPARCNEGOCIO"  => $parametros['NRPARCNEGOCIO'],
                    "DSDIRLOGOMARCA" => $file['path'],
                    "DSTYPELOGOTIPO" => $file['type'],
                    "QTSIZELOGOTIPO" => $file['size']
                );
                $this->businessPartnerFactory->sendImage($dataParam);
                $link = $file['path'];
            }
            $this->connection->commit();
            return $link;
        }
        catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteImage($parametros) {
        $caminhoAntigo = $parametros['LINK'];
        $this->FileServerService->deleteFile($caminhoAntigo);
        $dataParam = array(
          "NRPARCNEGOCIO" => $parametros['NRPARCNEGOCIO']
        );
        $this->businessPartnerFactory->deleteImage($dataParam);
    }

    public function getImagesLogotype($parametro) {
        return $this->businessPartnerFactory->getImagesLogotype($parametro);
    }

    public function getBusinessPartnerStandardFinancialIndices($params) {
        return $this->businessPartnerFactory->getBusinessPartnerStandardFinancialIndices($params);
    }

    public function salvarIndiceFinanceiroPadraoParceiro($nrindicespadrparc, $nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        if (empty($nrindicespadrparc)) {
            $this->inserirIndiceFinanceiroPadraoParceiro($nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $nrorg, $cdoperador);
        }
        else {
            $this->atualizarIndiceFinanceiroPadraoParceiro($cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $cdoperador, $nrorg, $nrindicespadrparc);
        }
    }

    private function inserirIndiceFinanceiroPadraoParceiro($nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $nrorg, $cdoperador) {
        $nrindicespadrparc = $this->newCode->getCodeByTableName("FCT_INDICESPADRPARC", $nrorg, 1, 10, true);
        $paramsInsert = array(
            "NRINDICESPADRPARC" => $nrindicespadrparc,
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "CDTPINDICE"        => $cdtpindice,
            "IDPERVALOR"        => $idpervalor,
            "CDINDICE"          => $cdindice,
            "NRPERCENTINDICE"   => $nrpercentindice,
            "NRORG"             => $nrorg,
            "IDATIVO"           => "S",
            "NRORGINCLUSAO"     => $nrorg,
            "CDOPERINCLUSAO"    => $cdoperador,
        );
        $this->businessPartnerFactory->inserirIndiceFinanceiroPadraoParceiro($paramsInsert);
    }

    private function atualizarIndiceFinanceiroPadraoParceiro($cdtpindice, $idpervalor, $cdindice, $nrpercentindice, $cdoperador, $nrorg, $nrindicespadrparc) {
        $paramsUpdate = array(
            "CDTPINDICE"        => $cdtpindice,
            "IDPERVALOR"        => $idpervalor,
            "CDINDICE"          => $cdindice,
            "NRPERCENTINDICE"   => $nrpercentindice,
            "CDOPERULTATU"      => $cdoperador,
            "NRORGULTATU"       => $nrorg,
            "NRINDICESPADRPARC" => $nrindicespadrparc,
            "NRORG"             => $nrorg
        );
        $this->businessPartnerFactory->atualizarIndiceFinanceiroPadraoParceiro($paramsUpdate);
    }

    public function excluirIndiceFinanceiroPadraoParceiro(array $nrindicespadrparc) {
        $nrorg = $this->environment->getNrOrg();
        $nrindicespadrparc = implode(",", $nrindicespadrparc);
        
        $this->businessPartnerFactory->excluirIndiceFinanceiroPadraoParceiro($nrindicespadrparc, $nrorg);
    }

    public function getSomaPercentualIndicesParceiro($nrparcnegocio, $nrindicespadrparc) {
        $params = array(
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "NRINDICESPADRPARC" => $nrindicespadrparc
        );
        $dataset = $this->businessPartnerFactory->getSomaPercentualIndicesParceiro($params);
        return (float)$dataset[0]["NRPERCENTINDICE"];
    }

    public function validarIndiceDuplicadoParceiro($nrparcnegocio, $nrindicespadrparc, $cdindice, $cdtpindice, $idpervalor) {
        $params = array(
            "NRPARCNEGOCIO"     => $nrparcnegocio,
            "CDTPINDICE"        => $cdtpindice,
            "IDPERVALOR"        => $idpervalor,
            "CDINDICE"          => $cdindice,
            "NRINDICESPADRPARC" => $nrindicespadrparc
        );
        $dataset = $this->businessPartnerFactory->validarIndiceDuplicadoParceiro($params);
        if ((int)$dataset[0]["EXISTE"] > 0) {
            throw new \Exception("O Índice Financeiro informado já está cadastrado para este Parceiro de Negócio.");
        }
    }

    public function inserirBairro($cdpais, $sgestado, $cdmunicipio, $nmbairro) {
        $nrorg = $this->environment->getNrOrg();
        $cdoperador = $this->environment->getCdOperador();

        $newCodeKey = "BAIRRO".$cdpais.$sgestado.$cdmunicipio;

        $cdbairro = $this->newCode->getCodeByTableName($newCodeKey, $nrorg);
        $params = array(
            "CDPAIS"         => $cdpais,
            "SGESTADO"       => $sgestado,
            "CDMUNICIPIO"    => $cdmunicipio,
            "CDBAIRRO"       => $cdbairro,
            "NMBAIRRO"       => $nmbairro,
            "NRORG"          => $nrorg,
            "NRORGINCLUSAO"  => $nrorg,
            "CDOPERINCLUSAO" => $cdoperador
        );
        $this->businessPartnerFactory->inserirBairro($params);

        return $cdbairro;
    }
}

?>