/* eslint-disable */
import {
    Metadata,
} from '@zeedhi/core';
import {
    DialogService, AlertService, FileInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-vuetify';

export class Utilities {
    public static showErrorMessage(error : any) {
        console.error(error);
        let msg = '';
        if (error.response && error.response.data && error.response.data.error) {
            msg = error.response.data.error;
        } else if (error.message) {
            msg = error.message;
        } else {
            msg = error;
        }
        if (!msg) {
            msg = 'Ocorreu um erro inesperado. Por favor, entre em contato com o suporte.';
        }

        this.showDialogMessage('error', 'Erro', msg);
    }

    public static showDialogMessage(type : string, title : string, text : string) {
        DialogService.show({
            name: type + '_dialog',
            type: type,
            title: title,
            text: text,
        });
    }

    public static showSuccessMessage() {
        AlertService.show({
            name: 'basic-alert',
            text: 'Processo realizado com sucesso!',
            top: true,
            left: false,
            color: 'success',
            timeout: 2000,
            buttonsSlot: [
                {
                  name: 'undo-button',
                  component: 'ZdButton',
                  icon: true,
                  iconName: 'close',
                  color: 'white',
                  events: {
                    click: () => { AlertService.hide(); },
                  },
                },
            ],
        });
    }

    public static showRequiredFieldsMessage() {
        this.showAlertMessage('Por favor, preencha todos os campos obrigatórios(*)!', 'error');
    }

    public static showAlertMessage(message: string, type: string) {
        AlertService.show({
            name: 'basic-alert',
            text: message,
            top: true,
            left: false,
            timeout: 2000,
            color: type,
        });
    }

    public static async buildArrayToUpload(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        if (file) {
          const base64file = await this.toBase64(file);
            return [{
                b64File: base64file,
                lastModified: file.lastModified,
                name: file.name,
                size: file.size,
                type: file.type,
                webkitRelativePath: file.webkitRelativePath,
            }];
        }
        return [];
    }

    /* public static async buildArrayToUploadPromise(nomeComponente: string) {
        const file = Metadata.getInstance<FileInput>(nomeComponente).value;
        return new Promise((resolve, reject) => {
            if (file) {
                this.toBase64(file).then((base64file) => {
                    resolve([{
                        b64File: base64file,
                        lastModified: file.lastModified,
                        name: file.name,
                        size: file.size,
                        type: file.type,
                        webkitRelativePath: file.webkitRelativePath,
                    }]);
                }).catch((error) => {
                    reject(error);
                });
            }
            else {
                resolve([]);
            }
        });
    }; */

    public static async toBase64(file: any) {
        return new Promise((resolve, reject) => {
            const reader = new FileReader();
            reader.readAsDataURL(file);
            reader.onload = () => resolve(reader.result);
            reader.onerror = (error) => reject(error);
        });
    }

    public static showConfirmDeleteMessage(event : any, functionConfirm : string) {
        event.preventDefault();
        functionConfirm = '{{' + functionConfirm + '}}';
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'button-no',
                    component: 'ZdButton',
                    label: 'Não',
                    events: {
                        click: () => DialogService.hide(),
                    },
                    outline: true,
                },
                {
                    name: 'button-yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: functionConfirm
                    },
                },
            ],
        });
    }
}
